/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdaminputhook.h"

typedef struct _GdamInputClass GdamInputClass;

struct _GdamInputClass {
	char* class_name;
	GdamInputAdjustmentHandler adj_handler;
	GdamInputRemoveHandler remove_handler;
};

static GHashTable* input_classes = NULL;

guint    gdam_input_hook_add_adjustment   (GtkAdjustment*    adjustment,
                                           const char*       input_class_name,
					   const char*       input_options)
{
	GdamInputClass* input_class;
	if (input_classes == NULL)
		return 0;
	input_class = g_hash_table_lookup(input_classes, input_class_name);
	if (input_class == NULL)
		return 0;
	if (input_class->adj_handler == NULL)
		return 0;
	return (*input_class->adj_handler)(adjustment, input_options);
}
void     gdam_input_hook_remove           (const char*       input_class_name,
                                           guint             hook_id)
{
	GdamInputClass* input_class;
	g_return_if_fail(input_classes != NULL);
	g_return_if_fail(hook_id != 0);
	input_class = g_hash_table_lookup(input_classes, input_class_name);
	g_return_if_fail(input_class != NULL);
	g_assert(input_class->remove_handler != NULL);
	return (*input_class->remove_handler)(hook_id);
}

void     gdam_input_hook_register_class   (const char*       input_class_name,
                                           GdamInputAdjustmentHandler handler,
					   GdamInputRemoveHandler remover)
{
	GdamInputClass* input_class;
	if (input_classes == NULL)
		input_classes = g_hash_table_new(g_str_hash, g_str_equal);
	g_return_if_fail(g_hash_table_lookup(input_classes, input_class_name)
	                                       == NULL);
	input_class = g_new(GdamInputClass, 1);
	input_class->class_name = g_strdup(input_class_name);
	input_class->adj_handler = handler;
	input_class->remove_handler = remover;
	g_hash_table_insert(input_classes,
	                    input_class->class_name,
			    input_class);
}
